// Hardware FPU example on the Tiva C Series launchpad
// Will modulate the PF2 and PF3 to produce in sinusoidal
// pattern.  Makes use of the hardware maths floating point library
#include "tilm4f120hqr.h"
#include <math.h>
#define MAX_DUTY 2047
void initPWM()
{
// LEDs are on PF1,PF2,PF3
// PF1 is connected to T0CCP1 (Timer 0)
// PF2 is connected to T1CCP0 (Timer 1)
// PF3 is connected to T1CCP1 (Timer 1)
// Turn on clock for GPIOF
	SYSCTL_RCGCGPIO |= BIT5;
	SYSCTL_GPIOHBCTL |= BIT5;      // turn on AHB access to GPIOF	
// Turn on Timer 0 and Timer 1
	SYSCTL_RCGCTIMER |= BIT0 + BIT1;
// Configure the GPIO pins appropriately
// Make bits 1,2,3 outputs 
	GPIOF_DIR |= BIT1 + BIT2 + BIT3 + BIT4;
// Turn on alternative function (function 7) for the pins
	GPIOF_AFSEL |= BIT1 + BIT2 + BIT3;
	GPIOF_PCTL &= ~((7 << 4) + (7 << 8) + (7 << 12));
	GPIOF_PCTL |= ((7 << 4) + (7 << 8) + (7 << 12));
	GPIOF_DEN |= BIT1+BIT2+BIT3+BIT4;
// Configure the Timers
//  Set preload value
	GPTM0_32_CTL = 0; // start by disabling the timers
	GPTM0_32_CFG = 4; // select 16 bit mode
	GPTM1_32_CTL = 0; // start by disabling the timers
	GPTM1_32_CFG = 4; // select 16 bit mode
	
	// Set T0AMS=1, T0ACMR = 0; T0AMR=2;
	GPTM0_32_TAMR = BIT3+BIT1;
	// Set T0BMS=1, T0BCMR = 0; T0BMR=2;
	GPTM0_32_TBMR = BIT3+BIT1;	
	// Set T1AMS=1, T1ACMR = 0; T1AMR=2;
	GPTM1_32_TAMR = BIT3+BIT1;
	// Set T1BMS=1, T1BCMR = 0; T1BMR=2;
	GPTM1_32_TBMR = BIT3+BIT1;	

// Set period and initial duty
	GPTM0_32_TAILR = MAX_DUTY+1;
	GPTM0_32_TBILR = MAX_DUTY+1;
	GPTM0_32_TAMATCHR = 0;
	GPTM0_32_TBMATCHR = 0;
	GPTM1_32_TAILR = MAX_DUTY+1;
	GPTM1_32_TBILR = MAX_DUTY+1;
	GPTM1_32_TAMATCHR = 0;
	GPTM1_32_TBMATCHR = 0;	
	
	// enable T0A and T0B with output inversion
	GPTM0_32_CTL |= BIT0+BIT8+BIT6+BIT14;
	// enable T1A and T1B with output inversion on TA only
	GPTM1_32_CTL |= BIT0+BIT8+BIT6;
	
}
void setDuty(int channel, int Duty)
{
	switch(channel)
	{
		case 0:{
			GPTM0_32_TBMATCHR = Duty;
			break;
		}
		case 1:{
			GPTM1_32_TAMATCHR = Duty;
			break;
		}
		case 2:{
			GPTM1_32_TBMATCHR = Duty;
			break;
		}
	}
}
void analogWrite(float Value)
{
	int Duty;
	Value = (Value + 1.0)*(float)MAX_DUTY/2;
	Duty = (int)Value;
	GPTM1_32_TAMATCHR = Duty;
	GPTM1_32_TBMATCHR = Duty;
}
void delay(int len)
{
	while(len--);
}
int main()
{
	float Value = 0;
	float Direction = 0.01;
	initPWM();
	while(1)
	{
		GPIOF_DATA |= BIT4;
		analogWrite(sinf(Value));
		GPIOF_DATA &= ~BIT4;
		Value = Value + Direction;
		if ( (Value == MAX_DUTY) || (Value == 0) )
			Direction = -Direction;
		delay(100000);
	}
}
